// $Id: EenaduParser.js,v 1.1.2.1 2005/05/26 22:27:52 vnagarjuna Exp $ -->

/* ***** BEGIN LICENSE BLOCK ***** 
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is Nagarjuna Venna.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * by the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Nagarjuna Venna <vnagarjuna@yahoo.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

//Parser for Eenadu Text - returns next syllable
//1. Remove all unnecessary symbols (talakattu, 0x2d)
//2..Put prefix symbols in the right place 
//3. Break up combo symbols if necessary
//Get this right first, make it fast later

function EenaduParser(input) {
    function removeRedundantSymbols(input)
    {
        var output = "";
        //1.
        for(var i = 0; i < input.length; ++i) {
            var str = input.substring(i, i + 1);
            if (!Eenadu.isRedundant(str))
                output += str;
        }
        return output;
    }

    this.index = 0;
    this.text = removeRedundantSymbols(input);
    this.length = this.text.length;
}

function EenaduParser_next()
{
    var syllable = "", prefix = "", stem = false, ctxt = Eenadu.ctxt_REGULAR;
    for(var remaining = this.length - this.index; remaining > 0; remaining = this.length - this.index) {
        var maxLookupLen = remaining > Eenadu.maxLookupLen ? Eenadu.maxLookupLen : remaining;
            
        var key = null, value = null;
        for(var i = maxLookupLen; i > 0; i--) {
            key = this.text.substring(this.index, this.index + i);
            //alert("looking up key " + key);
            value = Eenadu.lookup(key, ctxt);
            if (value != null) {
                //alert("match for key " + key + " , match = " + value);
                break;
            }
            else {
                //alert("No match");
            }
        }

        //No more match, return already matched text if available otherwise the key
        if (value == null) {
            if (syllable.length == 0 && prefix.length == 0) {
                //alert("returning key " + key);
                this.index += key.length;
                return key;
            }
            break;
        }

        //2.
        if (syllable.length == 0) {
            if (Eenadu.isPrefixSymbol(key))
                prefix += value;
            else {
                syllable += value;
                ctxt = Eenadu.getContext(key);
            }
        }
        else if (syllable.length != 0 && Eenadu.isSyllableStart(key)) {
            break;
        }
        else {
            syllable += value;
        }
        this.index += key.length;
    }

    //3.
    if (syllable.length != 0 || prefix.length != 0) {
        var output = "", specials = "";
        
        syllable += prefix;
        for(var i = 0; i < syllable.length; ++i) {
            var str = syllable.substring(i, i + 1);
            if (Eenadu.isSpecialValue(str))
                specials += str;
            else output += str;
        }
        //Hack
        if (specials.length == 2) {
            var str1 = specials.substring(0, 1);
            var str2 = specials.substring(1, 2);
            if ((str1 == Padma.vowelsn_E && str2 == Padma.vowelsn_AILEN) || 
                (str1 == Padma.vowelsn_AILEN && str2 == Padma.vowelsn_E))
                specials = Padma.vowelsn_AI;
        }
        output += specials;
        //alert("returning " + output);
        return output;
    }
    return null;
}
